import os
from casacore.images import image as casa_image
from PIL import Image
import subprocess
import time
import monotonic

FPS = 25

cmd = ["ffmpeg",
       "-f", "alsa", "-ac", "2", "-i", "hw:0,0",

       # stream image
       '-re',
       '-f', 'rawvideo',           # probably required for reading from stdin
       '-s', '1024x1024',          # should match image size
       '-pix_fmt', 'gray',
       '-i', '-',                  # read from stdin

       "-r", str(FPS),             # the framerate
       "-vcodec", "libx264",       # probably required for flv & rtmp

       "-preset", "ultrafast",     # the encoding quality preset
       "-g", "20",
       "-codec:a", "libmp3lame",   # mp3 for audio
       "-ar", "44100",             # 44k audio rate
       "-threads", "6",
       "-bufsize", "512k",
       "-f", "flv",                # required for rtmp
       {}                          # the rtmp stream url
       ]


def setup_stream_pipe(rtmp_url):
    """
    Setup a encoding process where you can pipe images to.

    args:
        rtmp_url (str): a rtmp url, for example rtmp://a.rtmp.youtube.com/live2/{SECRET}

    returns:
        subprocess.Popen: a subprocess pipe. Use pipe.stdin.write for writing images.
    """
    pipe = subprocess.Popen(cmd.format(rtmp_url), stdin=subprocess.PIPE)
    return pipe


def loop_images_in_path(path):
    """
    args:
        path (str): path to folder containing images

    returns:
        generator: yields casacore images
    """
    images = sorted([os.path.join(path, i) for i in os.listdir(path)])
    while True:
        for image_path in images:
            print(image_path)
            image = casa_image(image_path)
            data = image.getdata().squeeze()
            data -= data.min()
            data *= (255 / data.max())
            arr = data.astype('uint8')
            im = Image.fromarray(arr, 'L')
            return im.tobytes()


def stream(iterator, rtmp_stream):
    """
    stream the images returned by generated to rtmp server

    args:
        iterator (iterator): a image data iterator
        rtmp_stream (str): a rtmp url
    """
    pipe = setup_stream_pipe(rtmp_stream)
    for image_bytes in iterator:
        for i in range(FPS):
            pipe.stdin.write(image_bytes)
        duty_cycle = 1  # seconds
        time.sleep(duty_cycle - monotonic.monotonic() % duty_cycle)



import arthur
import unittest
from os import path

import arthur.imaging
import arthur.plot
import arthur.io

HERE = path.dirname(__file__)
VIS_PATH = path.join(HERE, 'data/2aartfaac.vis')


class TestArthur(unittest.TestCase):
    def test_arthur(self):
        start_time, body = arthur.io.read_data(VIS_PATH)
        img_data = arthur.imaging.full_calculation(body)
        arthur.plot.plot_image(start_time, img_data)


import struct
from datetime import datetime
import numpy as np
from arthur import constants


def parse_header(header):
    """
    parse a AARTFAAC visibilities triangle header.

    args:
        header (bytes): an AARTFAAC header

    returns:
        tuple: start date (datetime.time), body (numpy.array)

    """
    magic, _, start, end = struct.unpack("<IIdd", header[0:24])
    assert magic == constants.HDR_MAGIC
    return start, end


def read_data(handler):
    """
    read one data window from a file like object.
    args:
        handler: an object with a file like interface (read)
    returns:
        tuple (date, body)
    """
    raw_header = handler.read(constants.LEN_HDR)
    if len(raw_header) != constants.LEN_HDR:
        raise(IOError('end of file'))
    start_time, end_time = parse_header(raw_header)
    raw_body = np.fromfile(handler, count=(constants.LEN_BDY),
                           dtype=np.complex64)
    indices = create_indices(0, constants.NUM_CHAN, constants.NUM_BSLN,
                             constants.NUM_POLS)
    body = reshape_body(raw_body, indices, constants.NUM_CHAN,
                        constants.NUM_BSLN)
    date = datetime.utcfromtimestamp(start_time)
    return date, body


def read_full(path):
    """
    Open file in path and read until end.

    args:
        path (str): a path to a visibilities file
    returns:
        iterator
    """
    handler = open(path, 'rb')
    while True:
        try:
            yield read_data(handler)
        except IOError:
            raise StopIteration


def create_indices(pol, channels, baselines, pols):
    """
    Creates the indices to obtain an upper triangle of visibilities for a
    certain polarization from a data block as constructed via the GPU
    correlator.

    args:
        pol (int): probably polarisation
        channels (int): number of channels
        baselines (int): number of baselines
        pols (int): number of polarisations
    """
    I = np.empty((channels, baselines), dtype=np.uint32)
    for b in range(baselines):
        for c in range(channels):
            I[c, b] = pol + c * pols + b * pols * channels
    return I.reshape(I.size, 1)


def reshape_body(raw_body, indices, channels, baselines):
    """
    Restructure the data into CHANNELS x BASELINES matrix for a certain
    polarization.

    args:
        raw_body (numpy.array): a flat numpy array
        channels (int): number of channels
        baselines (int): number of baselines

    return:
        numpy.array
    """
    result = np.zeros((channels, baselines), dtype=np.complex64)
    np.copyto(result.reshape(result.size, 1), np.take(raw_body, indices))
    return result


import numpy as np
from arthur import constants
from functools import lru_cache


@lru_cache()
def load_antpos(path):
    """
    Load antenna positions and compute U,V coordinates

    args:
        path (str): path to antenna pos file

    returns:
        tuple: (numpy.array, numpy.array)
    """
    A = np.loadtxt(path)
    R = np.array([[-0.1195950000, -0.7919540000, 0.5987530000],
                  [0.9928230000, -0.0954190000, 0.0720990000],
                  [0.0000330000, 0.6030780000, 0.7976820000]])
    L = A.dot(R)

    U = np.zeros((constants.NUM_ANTS, constants.NUM_ANTS), dtype=np.float64)
    V = np.zeros((constants.NUM_ANTS, constants.NUM_ANTS), dtype=np.float64)

    for a1 in range(constants.NUM_ANTS):
        for a2 in range(constants.NUM_ANTS):
            U[a1, a2] = L[a1, 0] - L[a2, 0]
            V[a1, a2] = L[a1, 1] - L[a2, 1]
    return U, V


def grid(U, V, C, duv, size):
    G = np.zeros((size, size), np.complex64)
    G.fill(0)
    for a1 in range(constants.NUM_ANTS):
        for a2 in range(constants.NUM_ANTS):
            p = 1.0
            if a1 == a2:
                p = 0.5

            u = U[a1, a2] / duv + size / 2 - 1
            v = V[a1, a2] / duv + size / 2 - 1

            w = int(np.floor(u))
            e = int(np.ceil(u))
            s = int(np.floor(v))
            n = int(np.ceil(v))

            west_power = p - (u - w)
            east_power = p - (e - u)
            south_power = p - (v - s)
            north_power = p - (n - v)

            south_west_power = south_power * west_power
            north_west_power = north_power * west_power
            south_east_power = south_power * east_power
            north_east_power = north_power * east_power

            G[s, w] += south_west_power * C[a1, a2]
            G[n, w] += north_west_power * C[a1, a2]
            G[s, e] += south_east_power * C[a1, a2]
            G[n, e] += north_east_power * C[a1, a2]
    return G


def image(corr_mat, antposfile, f_hz, size):
    """
    Create an image from the correlation matrix
    """
    U, V = load_antpos(antposfile)
    mDuv = constants.C_MS / f_hz / 2.0
    gridvis = grid(U, V, corr_mat, mDuv, size)
    gridvis = np.fft.fftshift(gridvis)
    gridvis = np.flipud(np.fliplr(gridvis))
    gridvis = np.conjugate(gridvis)
    return np.real(np.fft.fftshift(np.fft.fft2(gridvis)))
